@@ -1,4 +1,4 @@
-function r = ranks(X,DIM,Mode);
+function r = ranks(X,DIM,Mode)
 % RANKS gives the rank of each element in a vector.
 % This program uses an advanced algorithm with averge effort O(m.n.log(n)) 
 % NaN in the input yields NaN in the output.
@@ -16,6 +16,11 @@ function r = ranks(X,DIM,Mode);
 % r = ranks(X,DIM,'advanced   ')
 %   implements an advanced algorithm with O(n*log(n)) computational 
 %   and O(n.log(n)) memory effort
+% r = ranks(X,DIM,'advanced-ties')
+%   implements an advanced algorithm with O(n*log(n)) computational 
+%   and O(n.log(n)) memory effort
+%   but without correction for ties 
+%   This is the fastest algorithm 
 %
 % see also: CORRCOEF, SPEARMAN, RANKCORR
 %
@@ -23,14 +28,14 @@ function r = ranks(X,DIM,Mode);
 % --
 
 
-%    $Id: ranks.m 5768 2009-05-06 12:49:40Z schloegl $
-%    Copyright (C) 2000-2002,2005 by Alois Schloegl <a.schloegl@ieee.org>	
+%    $Id$
+%    Copyright (C) 2000-2002,2005,2010,2013 by Alois Schloegl <alois.schloegl@gmail.com>	
 %    This script is part of the NaN-toolbox
-%    http://www.dpmi.tu-graz.ac.at/~schloegl/matlab/NaN/
+%       http://pub.ist.ac.at/~schloegl/matlab/NaN/
 
 %    This program is free software; you can redistribute it and/or modify
 %    it under the terms of the GNU General Public License as published by
-%    the Free Software Foundation; either version 2 of the License, or
+%    the Free Software Foundation; either version 3 of the License, or
 %    (at your option) any later version.
 %
 %    This program is distributed in the hope that it will be useful,
@@ -46,8 +51,8 @@ function r = ranks(X,DIM,Mode);
 % + computational effort is O(n.log(n)) instead of O(n^2)
 % + memory effort is O(n.log(n)), instead of O(n^2). 
 %     Now, the ranks of 8000 elements can be easily calculated
-% + NaN's in the input yield NaN in the output 
-% + compatible with this software and Matlab
+% + NaNs in the input yield NaN in the output 
+% + compatible with Octave and Matlab
 % + traditional method is also implemented for comparison. 
 
 
@@ -62,15 +67,24 @@ if isempty(Mode),
 	Mode='advanced   '; 
 end;
 
-sz = size(X);
+sz_orig = size (X);
+X = squeeze (X); %remove singleton dimensions for convenience
+nd = ndims (X);
 if (~DIM)
-	 [tmp,DIM] = min(find(sz>1));
-end;	 
-[N,M] = size(X);
-if (DIM==2),
-        X = X';
-	[N,M] = size(X);
-end; 
+	 DIM = 1;
+end
+if DIM > 1 %shift the array so that the dimension to sort over is first
+  perm = [DIM 1:(DIM-1) (DIM+1):nd];
+  X = permute (X, perm);
+end
+if nd > 2  %convert X to 2-D if it has >2 dimensions
+  sz = size(X);
+  N = sz(1);
+  M = prod(sz(2:end));
+  X = reshape(X, N, M);
+else
+  [N,M] = size(X);
+end
 
 if strcmp(Mode(1:min(11,length(Mode))),'traditional'), % traditional, needs O(m.n^2)
 % this method was originally implemented by: KH <Kurt.Hornik@ci.tuwien.ac.at>
@@ -79,22 +93,34 @@ if strcmp(Mode(1:min(11,length(Mode))),'traditional'), % traditional, needs O(m.
 r = zeros(size(X));
         for i = 1:M;
                 p = X(:, i(ones(1,N)));
-                r(:,i) = [(sum (p < p') + (sum (p == p') + 1) / 2)'];
+                r(:,i) = (sum (p < p') + (sum (p == p') + 1) / 2)';
         end;
         % r(r<1)=NaN;
         
-elseif strcmp(Mode(1:min(12,length(Mode))),'mtraditional'), % advanced
+elseif strcmp(Mode(1:min(12,length(Mode))),'mtraditional'), 
         % + memory effort is lower
         
 	r = zeros(size(X));
         for k = 1:N;
         for i = 1:M;
-                r(k,i) = [(sum (X(:,i) < X(k,i)) + (sum (X(:,i)  == X(k,i)) + 1) / 2)];
+                r(k,i) = (sum (X(:,i) < X(k,i)) + (sum (X(:,i)  == X(k,i)) + 1) / 2);
         end;
         end;
         % r(r<1)=NaN;
         
-elseif strcmp(Mode(1:min(11,length(Mode))),'advanced   '), % advanced
+elseif strcmp(Mode(1:min(13,length(Mode))),'advanced-ties'), % advanced
+        % + uses sorting, hence needs only O(m.n.log(n)) computations         
+        % - does not fix ties
+
+        r = zeros(size(X));
+        [sX, ix] = sort(X,1); 
+	for k=1:M,
+	        [tmp,r(:,k)] = sort(ix(:,k),1);	    % r yields the rank of each element 	
+	end;        
+	r(isnan(X)) = nan;
+
+        
+elseif strcmp(Mode(1:min(8,length(Mode))),'advanced'), % advanced
         % + uses sorting, hence needs only O(m.n.log(n)) computations         
         
         % [tmp,ix] = sort([X,Y]);     
@@ -141,12 +167,33 @@ elseif strcmp(Mode,'=='),
                 r1 = ranks(X,'traditional');
         end;
         if ~all(all(r==r1)),
-                fprintf(2,'WARNING RANKS: advanced algorithm does not agree with traditional one\n Please report to <a.schloegl@ieee.org>\n');
+                fprintf(2,'WARNING RANKS: advanced algorithm does not agree with traditional one\n Please report to <alois.schloegl@gmail.com>\n');
                 r = r1;
         end;
         r(isnan(X)) = nan;
 end;
 
-if (DIM==2)
-	r=r';
-end;	
+%reshape r to match the input X
+if nd > 2
+  r = reshape (r, sz);
+end
+if (DIM > 1)
+	r = ipermute (r, perm);
+end
+r = reshape (r, sz_orig); %restore any singleton dimensions
+
+
+%!shared z, r1, r2
+%! z = magic (4);
+%! r1 = [4   1   1   4; 2   3   3   2; 3   2   2   3; 1   4   4   1];
+%! r2 = [4   1   2   3; 1   4   3   2; 3   2   1   4; 2   3   4   1];
+%!assert (ranks(z), r1);
+%!assert (ranks(z, 2), r2);
+%! z = nan(2, 2, 2);  
+%! z(:, :, 1) = [1 2; 3 4];
+%! z(:, :, 2) = [4 3; 2 1];
+%! r1 = cat(3, [1 1; 2 2], [2 2; 1 1]);
+%! r2 = cat(3, [1 2; 1 2], [2 1; 2 1]);
+%!assert (ranks(z), r1);
+%!assert (ranks(z, 2), r2);
+%!assert (ranks(z, 3), r1);
