@@ -65,7 +65,7 @@ static void print_string_stdout(const char *s)
 	fputs(s,stdout);
 	fflush(stdout);
 }
-static void print_null(const char *s) {}
+static void print_null(const char*) {}
 
 static void (*liblinear_print_string) (const char *) = &print_string_stdout;
 
@@ -3073,7 +3073,7 @@ void destroy_param(parameter* param)
 		free(param->init_sol);
 }
 
-const char *check_parameter(const problem *prob, const parameter *param)
+const char *check_parameter(const problem*, const parameter *param)
 {
 	if(param->eps <= 0)
 		return "eps <= 0";
